#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pypungi']

package_data = \
{'': ['*']}

install_requires = \
['statsmodels', 'zerorpc']

setup(name='pypungi',
      version='0.11.15',
      description='Helper tool',
      author='James Otterson',
      author_email='jjotterson@gmail.com',
      url='https://github.com/jjotterson',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
