'''Helper tool
   import pypungi 
   pp = pypungi.app() #or pass tcp/ip
   or
   pp = pypungi.hub()()

'''

__version__ = "0.11.15"

from pypungi.main import hub 
from pypungi.app import app



if __name__ == '__main__':
    pass
    #import pypungi
    #
    #pp = pypungi.link()
    #pp.stash('hi')
    #pp.getStash()
    